﻿using System;
using System.Collections.Generic;
using System.Threading;

namespace ThreadWithCallback
{
    public delegate void ResultCallback(int result);
    class Program
    {
        private static readonly IList<int> Results = new List<int>();
        private static readonly object LockObject = new object();

        private static void ReportResults(int result)
        {
            lock (LockObject)
            {
                Results.Add(result);
            }
        }

        static void Main(string[] args)
        {
            Helper helper1 = new Helper(20, ReportResults);
            Thread t1 = new Thread(helper1.Run);

            Helper helper2 = new Helper(7, ReportResults);
            Thread t2 = new Thread(helper2.Run);

            //Thread t1 = new Thread(() => Helper.Run2(20, ReportResults));
            //Thread t2 = new Thread(() => Helper.Run2(7, ReportResults));

            t2.Start();
            t1.Start();

            t1.Join();
            t2.Join();
            Console.WriteLine(String.Join(", ", Results));
        }
    }

    public class Helper
    {
        private readonly int _number;
        private readonly ResultCallback _reportResult;

        public Helper(int number, ResultCallback callbackMethod)
        {
            _number = number;
            _reportResult = callbackMethod;
        }

        public void Run()
        {
            for (int i = 0; i < 100; i++)
            {
                if (i % _number == 0)
                {
                    _reportResult(i);
                    Thread.Sleep(1);
                }
            }
        }

        public static void Run2(int number, ResultCallback callback)
        {
            for (int i = 0; i < 100; i++)
            {
                if (i % number == 0)
                {
                    callback(i);
                    Thread.Sleep(1);
                }
            }
        }
    }
}
